# -*- coding: utf-8 -*-

from __future__ import absolute_import, unicode_literals

from .version import VERSION
from .default_consumer import DefaultConsumer
from .debug_consumer import DebugConsumer
from .buffered_consumer import BufferedConsumer
from .async_buffered_consumer import AsyncBufferedConsumer
from .consumer import Consumer
from .message import CustomEventMessage

__version__ = VERSION


class GrowingTracker(object):

    def __init__(self, product_id, consumer=None):
        self._product_id = product_id
        self._consumer = consumer or DefaultConsumer(product_id)

    @staticmethod
    def consumer(consumer):
        return GrowingTracker(consumer._product_id, consumer)

    def track_custom_event(self, event_name, login_user_id, event_time=None, domain=None, num=None,
                           attributes=None):
        message = CustomEventMessage.EventBuilder() \
            .set_product_id(self._product_id).set_event_name(event_name).set_event_time(event_time) \
            .set_domain(domain).set_login_user_id(login_user_id).set_num(num).set_attributes(
            attributes).build()
        self._consumer.send(message)

    def track(self, message):
        message.set_product_id(self._product_id)
        self._consumer.send(message)



# -*- coding: utf-8 -*-

import time


class CustomEventMessage(object):
    def __init__(self, event_time, event_name, login_user_id, domain, num, attributes, product_id=None):
        self.event_type = "cstm"  # 1
        self.product_id = product_id
        self.event_time = event_time
        self.event_name = event_name
        self.domain = domain
        self.login_user_id = login_user_id
        self.num = num
        self.attributes = attributes

    def set_data_source_id(self, data_source_id):
        self.data_source_id = data_source_id

    def set_product_id(self, product_id):
        self.product_id = product_id

    def __str__(self):
        message_str = '{event_type:' + self.event_type
        message_str += ',event_name:' + self.event_name
        message_str += ',product_id:' + self.product_id
        message_str += ',event_time:' + str(self.event_time)
        if self.num is not None:
            message_str += ',num:' + str(self.num)
        if self.domain is not None:
            message_str += ',domain:' + self.domain
        if self.login_user_id is not None:
            message_str += ',login_user_id:' + self.login_user_id
        if self.attributes is not None:
            message_str += ',attributes:' + str(self.attributes)
        message_str += '}'
        return message_str

    class EventBuilder(object):
        def __init__(self):
            self.product_id = None
            self.event_time = int(time.time() * 1000)
            self.event_name = None
            self.domain = None
            self.num = None
            self.login_user_id = None
            self.attributes = None
            pass

        def set_num(self, num):
            self.num = num
            return self

        def set_product_id(self, product_id):
            self.product_id = product_id
            return self

        def set_event_time(self, timestamp):
            if timestamp is not None:
                self.event_time = timestamp
            return self

        def set_event_name(self, name):
            self.event_name = name
            return self

        def set_domain(self, device_id):
            self.domain = device_id
            return self

        def set_login_user_id(self, user_id):
            self.login_user_id = user_id
            return self

        def add_attribute(self, key, value):
            if self.attributes is None:
                self.attributes = {}
            self.attributes[key] = value
            return self

        def set_attributes(self, map):
            self.attributes = map
            return self

        def build(self):
            return CustomEventMessage(self.event_time, self.event_name,
                                      self.login_user_id, self.domain, self.num, self.attributes,
                                      self.product_id)



# -*- coding: utf-8 -*-

from __future__ import absolute_import, unicode_literals
import json
import time
from .version import VERSION


class DataParser(object):
    def get_bytes(self, messages, stm=None):
        pass

    def get_params(self, stm=None):
        pass

    def get_headers(self):
        pass

    @staticmethod
    def get_timestamp():
        return int(time.time() * 1000)


class JsonParser(DataParser):

    def __init__(self):
        pass

    def get_bytes(self, messages, stm=None):
        batch = []
        for message in messages:
            batch.append(self.json(message))
        batch_json = '[{0}]'.format(','.join(batch))
        return batch_json

    def get_headers(self):
        return {'content-type': 'application/json'}

    def get_params(self, stm=None):
        return {'stm': stm}

    @staticmethod
    def json_dumps(data, cls=None):
        return json.dumps(data, separators=(',', ':'), cls=cls)

    def json(self, message):
        dict = {}

        # event message
        if hasattr(message, 'event_type') and message.event_type is not None:
            dict['t'] = message.event_type
        if hasattr(message, 'event_time') and message.event_time is not None:
            dict['tm'] = message.event_time
        if hasattr(message, 'login_user_id') and message.login_user_id is not None:
            dict['cs1'] = message.login_user_id
        if hasattr(message, 'event_name') and message.event_name is not None:
            dict['n'] = message.event_name
        if hasattr(message, 'num') and message.num is not None:
            dict['num'] = message.num
        if hasattr(message, 'attributes') and message.attributes is not None:
            dict['var'] = message.attributes

        dict['av'] = VERSION
        dict['b'] = 'python'

        return self.json_dumps(dict)
